from collections.abc import Iterable
import csv

from django.conf import settings
from django.contrib.auth import get_user_model
from django.contrib.auth.mixins import PermissionRequiredMixin
from django.core.exceptions import ObjectDoesNotExist
from django.db.models import Q
from django.http import HttpResponse
from django.views.generic import ListView

from wafer.talks.models import (
 ACCEPTED, PROVISIONAL, SUBMITTED, UNDER_CONSIDERATION, Talk)

from bursary.models import Bursary
from invoices.models import Invoice
from register.dates import nights
from register.models import (
    Accomm, AccommNight, Attendee, ChildCare, Meal, Visa)
from register.views import STEPS

ALL_STEPS = len(STEPS) - 1


class CSVExportView(ListView):
    """Export the given columns for the model as CSV."""
    columns = None
    filename = None

    def get_data_line(self, instance):
        ret = []
        for column in self.columns:
            obj = instance
            for component in column.split('.'):
                try:
                    obj = getattr(obj, component)
                except ObjectDoesNotExist:
                    obj = '%s missing!' % component
                    break
                except AttributeError:
                    obj = getattr(self, component)(obj)
                if not obj:
                    break
                if callable(obj):
                    obj = obj()
            if (not isinstance(obj, (str, bytes))
                    and isinstance(obj, Iterable)):
                ret.append(str([str(x) for x in obj]))
            else:
                ret.append(str(obj))

        return ret

    def write_rows(self, writer, objects):
        for instance in objects:
            writer.writerow(self.get_data_line(instance))

    def render_to_response(self, context, **response_kwargs):
        response = HttpResponse(content_type='text/csv; charset=utf-8')
        response['Content-Disposition'] = (
            'attachment; filename="%s"' % self.filename
        )

        writer = csv.writer(response)
        writer.writerow(self.columns)
        self.write_rows(writer, context['object_list'])

        return response


class AttendeeBadgeExport(PermissionRequiredMixin, CSVExportView):
    name = 'Attendee badges'
    model = Attendee
    filename = "attendee_badges.csv"
    ordering = ('user__username',)
    columns = [
        'user.username', 'confirmed', 'user.email', 'user.get_full_name',
        'nametag_2', 'nametag_3', 'languages', 'food.diet',
    ]
    permission_required = 'register.view_attendee'

    def get_queryset(self):
        qs = super().get_queryset()
        return qs.filter(completed_register_steps=ALL_STEPS)


class AttendeeAccommExport(PermissionRequiredMixin, CSVExportView):
    name = 'Accommodation'
    model = Accomm
    filename = "attendee_accommodation.csv"
    ordering = ('attendee__user__username',)
    columns = [
        'attendee.user.username', 'attendee.user.get_full_name',
        'attendee.user.email', 'attendee.confirmed', 'attendee.paid',
        'attendee.user.attendee.arrived',
        'accomm_bursary', 'attendee.gender',
        'attendee.country', 'option', 'requirements',
        'family_usernames', 'get_checkin_checkouts', 'room',
    ]
    permission_required = 'register.view_accomm'

    def get_queryset(self):
        qs = super().get_queryset()
        return qs.filter(attendee__completed_register_steps=ALL_STEPS)

    def accomm_bursary(self, accomm):
        try:
            bursary = accomm.attendee.user.bursary
        except ObjectDoesNotExist:
            return ''
        if not bursary.request_accommodation:
            return ''
        return bursary.accommodation_status


class AccommNightsExport(PermissionRequiredMixin, CSVExportView):
    name = 'Accommodation Nights'
    model = AccommNight
    filename = "accommodation_nights.csv"
    columns = [
        'night', 'option', 'paid', 'bursary_accepted', 'bursary_pending',
        'bursary_submitted', 'unknown',
    ]
    permission_required = 'register.view_accomm'

    def write_rows(self, writer, objects):
        for night in nights(orga=True):
            for option in settings.PRICES['accomm'].keys():
                writer.writerow(self.get_night_data(night, option))

    def get_night_data(self, night, option):
        accomm_night = AccommNight.objects.get(date=night)
        paid = 0
        unknown = 0
        bursaried = {
            'accepted': 0,
            'pending': 0,
            'submitted': 0,
        }
        for accomm in accomm_night.accomm_set.filter(option=option):
            if not accomm.attendee.user.userprofile.is_registered():
                continue
            try:
                bursary = accomm.attendee.user.bursary
            except ObjectDoesNotExist:
                bursary = None
            if bursary and bursary.potential_bursary('accommodation'):
                bursaried[bursary.accommodation_status] += 1
            elif accomm.attendee.paid():
                paid += 1
            else:
                unknown += 1
        return [night, option, paid, bursaried['accepted'],
                bursaried['pending'], bursaried['submitted'], unknown]


class ChildCareExport(PermissionRequiredMixin, CSVExportView):
    name = 'Childcare'
    model = ChildCare
    filename = "attendee_child_care.csv"
    ordering = ('attendee__user__username',)
    columns = [
        'attendee.user.username', 'attendee.user.get_full_name',
        'attendee.user.email', 'attendee.confirmed', 'attendee.paid',
        'attendee.user.bursary.accommodation_status',
        'attendee.arrival', 'attendee.departure',
        'needs', 'details',
    ]
    permission_required = 'register.view_childcare'

    def get_queryset(self):
        qs = super().get_queryset()
        return qs.filter(attendee__completed_register_steps=ALL_STEPS)


class TalksExport(PermissionRequiredMixin, CSVExportView):
    name = 'Talks'
    model = Talk
    permission_required = 'talks.edit_private_notes'
    filename = "talk_evaluations.csv"
    ordering = ("talk_id",)
    columns = [
        'talk_id', 'title', 'speaker_emails', 'abstract', 'talk_type.name',
        'track.name', 'get_status_display', 'submission_time',
        'speaker_genders', 'review_score', 'review_count', 'notes',
        'private_notes', 'all_review_comments', 'speakers_registered',
        'speakers_confirmed',
    ]

    def _speaker_email(self, user):
        name = user.userprofile.display_name()
        if "," in name:
            name = f'"{name}"'
        return f"{name} <{user.email}>"

    def speaker_emails(self, talk):
        authors = list(talk.authors.all())
        # Corresponding authors first
        authors.sort(
            key=lambda author: '' if author == talk.corresponding_author
                    else author.userprofile.display_name())
        emails = [self._speaker_email(author) for author in authors]
        return ", ".join(emails)

    def speaker_genders(self, talk):
        return "+".join(sorted(set(
            author.attendee.gender or "?"
            for author in talk.authors.filter(attendee__isnull=False)
        )))

    def all_review_comments(self, talk):
        return [
            "(%s) %s" % (review.reviewer.username, review.notes.raw)
            for review in talk.reviews.all()
            if review.notes.raw
        ]

    def _summarize_bools(self, items):
        """Render a list of booleans into one of (True, False, some)"""
        if all(items):
            return True
        if not any(items):
            return False
        return "some"

    def speakers_registered(self, talk):
        registered = [speaker.userprofile.is_registered() for speaker in talk.authors.all()]
        return self._summarize_bools(registered)

    def speakers_confirmed(self, talk):
        # is_registered assures that Attendee exists
        confirmed = [speaker.userprofile.is_registered() and speaker.attendee.confirmed()
                     for speaker in talk.authors.all()]
        return self._summarize_bools(confirmed)


class SpeakersRegistrationExport(PermissionRequiredMixin, CSVExportView):
    name = 'Speaker Registration'
    model = get_user_model()
    permission_required = 'talks.edit_private_notes'
    filename = "speakers_registration.csv"
    columns = [
        'username',
        'userprofile.display_name',
        'accepted_talks',
        'provisional_talks',
        'under_consideration_talks',
        'total_talks',
        'registered',
        'attendee.confirmed',
        'attendee.final_dates',
        'attendee.arrival',
        'attendee.departure',
        'bursary.need',
        'travel_status',
        'food_status',
        'accommodation_status',
        'expenses_status',
    ]

    def get_queryset(self):
        qs = super().get_queryset()
        return qs.filter(
            talks__status__in=(ACCEPTED, PROVISIONAL, SUBMITTED, UNDER_CONSIDERATION)
        ).distinct()

    def registered(self, speaker):
        return speaker.userprofile.is_registered()

    def accepted_talks(self, speaker):
        return speaker.talks.filter(status=ACCEPTED).count()

    def provisional_talks(self, speaker):
        return speaker.talks.filter(status=PROVISIONAL).count()

    def total_talks(self, speaker):
        return speaker.talks.filter(
            status__in=(ACCEPTED, PROVISIONAL, SUBMITTED, UNDER_CONSIDERATION)
        ).count()

    def under_consideration_talks(self, speaker):
        return speaker.talks.filter(status=UNDER_CONSIDERATION).count()

    def _bursary_status(self, speaker, key):
        try:
            bursary = speaker.bursary
        except Bursary.DoesNotExist:
            return "not requested"
        if not getattr(bursary, f"request_{key}"):
            return "not requested"
        return getattr(bursary, f"{key}_status")

    def accommodation_status(self, speaker):
        return self._bursary_status(speaker, "accommodation")

    def expenses_status(self, speaker):
        return self._bursary_status(speaker, "expenses")

    def food_status(self, speaker):
        return self._bursary_status(speaker, "food")

    def travel_status(self, speaker):
        return self._bursary_status(speaker, "travel")


class FoodExport(PermissionRequiredMixin, CSVExportView):
    name = 'Food'
    model = Meal
    filename = "meals.csv"
    ordering = ('date', 'meal',)
    columns = [
        'date', 'meal', 'total', 'total_unconfirmed',
        'regular', 'vegetarian', 'vegan', 'gluten_free',
        'other', 'other_details',
    ]
    permission_required = 'register.view_food'

    def render_to_response(self, context, **response_kwargs):
        self._confirmed_attendees = {}
        return super().render_to_response(context, **response_kwargs)

    def attendee_confirmed(self, attendee_id):
        if attendee_id not in self._confirmed_attendees:
            attendee = Attendee.objects.get(id=attendee_id)
            self._confirmed_attendees[attendee_id] = attendee.confirmed()
        return self._confirmed_attendees[attendee_id]

    def get_data_line(self, meal):
        row = {
            'date': meal.date.isoformat(),
            'meal': meal.meal,
            'total': 0,
            'total_unconfirmed': meal.food_set.count(),
            'regular': 0,
            'vegetarian': 0,
            'vegan': 0,
            'gluten_free': 0,
            'other': 0,
            'other_details': [],
        }

        for food in meal.food_set.all():
            if not self.attendee_confirmed(food.attendee_id):
                continue

            diet = food.diet
            if diet == '':
                diet = 'regular'
            elif diet == 'other':
                details = [food.attendee.user.username]
                details.append(food.special_diet)
                row['other_details'].append(': '.join(details))
            row[diet] += 1
            row['total'] += 1

        row['other_details'] = ', '.join(row['other_details'])
        return [row.get(key) for key in self.columns]


class SpecialDietExport(PermissionRequiredMixin, CSVExportView):
    name = 'Special diets'
    filename = "diets.csv"
    ordering = ('attendee__user__username',)
    columns = [
        'username', 'name', 'confirmed', 'diet', 'special_diet'
    ]
    permission_required = 'register.view_food'

    def get_queryset(self):
        return Meal.objects.get(**self.kwargs).food_set.exclude(
                special_diet__exact='',
                diet__exact='',
                attendee__completed_register_steps__lt=ALL_STEPS,
            ).select_related(
                'attendee',
                'attendee__user',
                'attendee__user__bursary',
                'attendee__user__userprofile',
            ).order_by(*self.ordering)

    def get_data_line(self, food):
        row = {
            'username': food.attendee.user.username,
            'name': food.attendee.user.userprofile.display_name(),
            'confirmed': food.attendee.confirmed(),
            'diet': food.diet,
            'special_diet': food.special_diet,
        }
        return [row.get(key) for key in self.columns]

    def write_rows(self, writer, objects):
        super().write_rows(writer, objects)
        count = {True: 0, False: 0}
        for food in Meal.objects.get(**self.kwargs).food_set.filter(
                special_diet__exact='',
                diet__exact='',
            ).select_related(
                'attendee',
                'attendee__user',
                'attendee__user__userprofile',
            ):
            count[self.attendee_confirmed(food.attendee)] += 1
        for confirmed in (True, False):
            row = {
                'username': '*',
                'name': 'Everyone Else - {} people'.format(count[confirmed]),
                'confirmed': confirmed,
                'diet': '',
                'special_diet': '',
            }
            writer.writerow([row.get(key) for key in self.columns])


class BursaryExport(PermissionRequiredMixin, CSVExportView):
    name = 'Bursaries'
    filename = "bursaries.csv"

    columns = [
        'user.username',
        'user.userprofile.display_name',
        'user.email',
        'user.attendee.arrived',
        'user.attendee.country',
        'travel_from',
        'travel_status',
        'food_status',
        'accommodation_status',
        'expenses_status',
    ]
    permission_required = 'register.view_attendee'

    approved = ('pending', 'accepted',)

    def get_queryset(self):
        return Bursary.objects.filter(
            Q(travel_status__in=self.approved)
            | Q(food_status__in=self.approved)
            | Q(accommodation_status__in=self.approved)
            | Q(expenses_status__in=self.approved)
        ).exclude(
            user__attendee__id=None, # exclude unregistered people
        ).prefetch_related(
            'user',
            'user__attendee',
            'user__userprofile',
        ).order_by('user__username')


class FingerprintExport(PermissionRequiredMixin, CSVExportView):
    name = 'Fingerprints'
    filename = 'fingerprints.csv'

    queryset = Attendee.objects.exclude(pgp_fingerprints='')
    columns = (
        'user.username',
        'user.userprofile.display_name',
        'user.email',
        'pgp_fingerprints',
        'keysigning_id',
    )
    permission_required = 'register.view_attendee'

    def get_queryset(self):
        qs = super().get_queryset()
        return qs.filter(completed_register_steps=ALL_STEPS)


class InvoiceExport(PermissionRequiredMixin, CSVExportView):
    name = 'Invoices'
    filename = 'invoices.csv'

    queryset = Invoice.objects.all()
    ordering = ('reference_number',)
    columns = (
        'reference_number',
        'status',
        'date',
        'last_update',
        'recipient.username',
        'recipient.userprofile.display_name',
        'compound',
        'transaction_id',
        'total',
    )
    permission_required = 'invoices.view_invoice'


class VisaExport(PermissionRequiredMixin, CSVExportView):
    name = 'Visas'
    filename = 'visas.csv'

    queryset = Visa.objects.all()
    columns = (
        'attendee.user.username',
        'attendee.user.userprofile.display_name',
        'attendee.user.email',
        'country',
        'travel_bursary_status',
        'accommodation_bursary_status',
        'food_bursary_status',
        'attendance_approval_status',
    )
    permission_required = 'register.view_visa'

    def get_queryset(self):
        qs = super().get_queryset()
        return qs.filter(attendee__completed_register_steps=ALL_STEPS)

    def _bursary_status(self, visa, attribute, non_existent_value=''):
        try:
            bursary = visa.attendee.user.bursary
        except ObjectDoesNotExist:
            return non_existent_value
        if attribute != 'attendance':
            if not getattr(bursary, f'request_{attribute}'):
                return non_existent_value
        return getattr(bursary, f'{attribute}_status')

    def travel_bursary_status(self, visa):
        return self._bursary_status(visa, 'travel')

    def accommodation_bursary_status(self, visa):
        return self._bursary_status(visa, 'accommodation')

    def food_bursary_status(self, visa):
        return self._bursary_status(visa, 'food')

    def attendance_approval_status(self, visa):
        return self._bursary_status(visa, 'attendance')


class SecurityInfoExport(PermissionRequiredMixin, CSVExportView):
    name = 'Security Info'
    model = Attendee
    filename = "attendee_security_info.csv"
    ordering = ('user__username',)
    columns = [
        'user.username', 'user.email', 'full_legal_name', 'date_of_birth',
        'country_of_citizenship', 'confirmed', 'arrival', 'departure',
    ]
    permission_required = 'register.view_security_info'

    def get_queryset(self):
        qs = super().get_queryset()
        return qs.filter(completed_register_steps=ALL_STEPS)



from debconf.common_settings import *

INSTALLED_APPS = (
    'badges',
    'daytrip',
    'debconf',
    'exports',
    'bursary',
    'front_desk',
    'invoices',
    'register',
) + INSTALLED_APPS

BAKERY_VIEWS += (
    'register.views.statistics.StatisticsView',
)



from django.conf import settings
from django.contrib.auth.mixins import LoginRequiredMixin
from django.core.exceptions import BadRequest, PermissionDenied
from django.http import Http404
from django.db import transaction
from django.urls import reverse
from django.views.generic import TemplateView
from django.views.generic.edit import UpdateView

from register.models.attendee import Attendee
from register.models.queue import Queue
from register.views import STEPS

from daytrip.forms import TravelInsuranceForm
from daytrip.models import TravelInsurance


class DayTripView(LoginRequiredMixin, TemplateView):
    template_name = 'daytrip/daytrip.html'

    def get_context_data(self, **kwargs):
        if 'daytrip' not in settings.PRICES:
            raise Http404
        user = self.request.user
        try:
            user.attendee
        except Attendee.DoesNotExist:
            raise PermissionDenied("Not yet registered")
        context = super().get_context_data(**kwargs)
        options = list(self.get_options())
        context['options'] = options
        context['registered_option'] = None
        context['invoice_register_step'] = f'register-step-{len(STEPS) - 2}'
        for option in options:
            if option['position']:
                context['registered_option'] = option
        context['DEBCONF_BILLING_CURRENCY_SYMBOL'] = (
            settings.DEBCONF_BILLING_CURRENCY_SYMBOL)
        context['DAYTRIP_OPEN'] = settings.DAYTRIP_OPEN
        context['DAYTRIP_INSURANCE_OPEN'] = settings.DAYTRIP_INSURANCE_OPEN
        return context

    def get_options(self):
        for key, option in settings.PRICES['daytrip'].items():
            queue = self.get_queue(key)
            attendees = [slot.attendee for slot in queue.slots.all()]
            slot = queue.slots.filter(attendee=self.request.user.attendee)
            position = None
            if slot.exists():
                position = slot.first().position
            yield {
                'id': key,
                'description': option['description'],
                'long_description': option['long_description'],
                'details_pdf': option.get('details_pdf'),
                'price': option['price'],
                'closed': option.get('closed', False),
                'insurance_price': option['insurance_price'],
                'position': position,
                'capacity': option['capacity'],
                'total_registered': len(attendees),
                'attendees': attendees,
            }

    def get_queue(self, key):
        return Queue.objects.get_or_create(name=f'DayTrip {key}')[0]

    def register(self, key, attendee):
        self.get_queue(key).slots.get_or_create(attendee=attendee)

    def unregister(self, key, attendee):
        self.get_queue(key).slots.filter(attendee=attendee).delete()

    @transaction.atomic
    def post(self, request, *args, **kwargs):
        if not settings.DAYTRIP_OPEN:
            raise BadRequest("Registration is closed")
        try:
            daytrip = request.POST['daytrip']
            action = request.POST['action']
        except KeyError as e:
            raise BadRequest(e)
        try:
            attendee = request.user.attendee
        except Attendee.DoesNotExist:
            raise PermissionDenied("Not yet registered")
        if daytrip not in settings.PRICES['daytrip']:
            raise BadRequest("Unknown daytrip")
        if settings.PRICES['daytrip'][daytrip].get('closed', False):
            raise BadRequest("Daytrip registration is closed")
        if action == 'register':
            # Ensure we aren't double-registered
            for key in settings.PRICES['daytrip']:
                if key != daytrip:
                    self.unregister(key, attendee)
            self.register(daytrip, attendee)
        elif request.POST['action'] == 'unregister':
            self.unregister(daytrip, attendee)
        else:
            raise BadRequest("Unknown action")
        return self.get(request, *args, **kwargs)


class InsuranceView(UpdateView):
    form_class = TravelInsuranceForm

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['DAYTRIP_INSURANCE_OPEN'] = settings.DAYTRIP_INSURANCE_OPEN
        return context

    def get_object(self, queryset=None):
        if 'daytrip' not in settings.PRICES:
            raise Http404
        attendee = self.request.user.attendee
        daytrip_option = attendee.daytrip_option
        if 'insurance_price' not in settings.PRICES['daytrip'][daytrip_option]:
            raise Http404
        try:
            return attendee.travel_insurance
        except TravelInsurance.DoesNotExist:
            return TravelInsurance(attendee=self.request.user.attendee)

    def post(self, request, *args, **kwargs):
        if not settings.DAYTRIP_INSURANCE_OPEN:
            raise BadRequest("Registration is closed")
        return super().post(request, *args, **kwargs)

    def get_success_url(self):
        return reverse('day-trip')



# Generated by Django 4.2.11 on 2024-03-27 01:40

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("register", "0015_accomm_option"),
    ]

    operations = [
        migrations.AddField(
            model_name="attendee",
            name="affiliation",
            field=models.TextField(blank=True),
        ),
    ]



from django.db import models
from django.core.validators import RegexValidator
from django.contrib.auth import get_user_model
from django.utils.translation import gettext_lazy as _
from django.utils.translation import pgettext_lazy
from django_countries.fields import CountryField

from wafer.schedule.models import ScheduleBlock
from debconf.countries import Countries
from debconf.models import GENDERS


INVOLVEMENT_LEVELS = (
    (0, _('Beginner')),
    (1, _('User')),
    (2, _('Contributor')),
    (4, _('Debian Maintainer (DM)')),
    (5, _('Debian Developer (DD)')),
)


class ShirtSize(models.Model):
    description = models.CharField(
        max_length=32,
        verbose_name=_("Description"))

    def __str__(self):
        return self.description


class Diet(models.Model):
    option = models.CharField(
        max_length=32,
        verbose_name=_("Diet"))

    def __str__(self):
        return self.option


class RegistrationType(models.Model):
    description = models.CharField(
        max_length=64,
        verbose_name=_("Description"),
    )

    def __str__(self):
        return self.description

class Registration(models.Model):
    class Meta:
        verbose_name = pgettext_lazy('conference', 'registration')
        verbose_name_plural = pgettext_lazy('conference', 'registrations')

    user = models.OneToOneField(get_user_model(), on_delete=models.CASCADE)
    date = models.DateTimeField(auto_now_add=True)
    phone_number = models.CharField(
        max_length=24,
        null=True,
        blank=True,
        validators=[RegexValidator(regex=r"[^0-9() +-]", inverse_match=True)],
    )
    registration_type = models.ForeignKey(
        RegistrationType,
        on_delete=models.PROTECT,
        null=True,
        blank=True,
    )
    involvement = models.IntegerField(
        null=True,
        blank=True,
        choices=INVOLVEMENT_LEVELS,
        verbose_name=_("Level of involvement with Debian"),
    )
    gender = models.CharField(
        max_length=1,
        blank=True,
        null=True,
        choices=GENDERS.items(),
        verbose_name=_("Gender"),
    )
    country = CountryField(
        countries=Countries,
        null=True,
        blank=True,
        verbose_name=_("Country"),
    )
    city_state = models.CharField(
        max_length=128,
        blank=True,
        verbose_name=_("City/State or Province")
    )
    shirt_size = models.ForeignKey(
        ShirtSize,
        on_delete=models.PROTECT,
        null=True,
        blank=True,
        verbose_name=_("Shirt size"),
    )

    arranged_accommodation = models.BooleanField(
        default=False,
        verbose_name=_("Conference-arranged accommodation"),
    )
    arranged_food = models.BooleanField(
        default=False,
        verbose_name=_("Conference-arranged food"),
    )
    diet = models.ForeignKey(
        Diet,
        on_delete=models.PROTECT,
        null=True,
        blank=True,
        verbose_name=_("Diet"),
    )

    travel_reimbursement = models.BooleanField(
        default=False,
        verbose_name=_("Reimburse travel costs"),
    )
    travel_cost = models.IntegerField(
        default=None,
        null=True,
        blank=True,
        verbose_name=_("Travel costs"),
    )
    notes = models.TextField(
        blank=True,
        null=True,
        verbose_name=_("Notes"),
    )

    # attendance info
    days =  models.ManyToManyField(
        ScheduleBlock,
        verbose_name=_('Which days you will attend'),
    )

    @property
    def full_name(self):
        if self.user:
            return self.user.get_full_name()
        else:
            return None


def is_registered(user):
    return Registration.objects.filter(user=user).exists()



from django.contrib import admin

from daytrip.models import TravelInsurance


class TravelInsuranceAdmin(admin.ModelAdmin):
    list_display = ('username', 'full_name', 'daytrip_option')
    search_fields = ('reference_number', 'attendee__user__username',
                     'attendee_user__first_name', 'attendee__user__last_name')

    def username(self, travel_insurance):
        return travel_insurance.attendee.user.username

    def full_name(self, travel_insurance):
        return travel_insurance.attendee.user.get_full_name()

    def daytrip_option(self, travel_insurance):
        return travel_insurance.attendee.daytrip_option


admin.site.register(TravelInsurance, TravelInsuranceAdmin)



from django.urls import re_path

from exports.views import (
    AccommNightsExport, AttendeeAccommExport, AttendeeBadgeExport,
    BursaryExport, ChildCareExport, FingerprintExport, FoodExport,
    InvoiceExport, SecurityInfoExport, SpeakersRegistrationExport,
    SpecialDietExport, TalksExport, VisaExport,
)

urlpatterns = [
    re_path(r'^attendees/admin/export/accomm/$', AttendeeAccommExport.as_view(),
        name='attendee_admin_export_accomm'),
    re_path(r'^attendees/admin/export/accomm_nights/$', AccommNightsExport.as_view(),
        name='attendee_admin_export_accomm_nights'),
    re_path(r'^attendees/admin/export/badges/$', AttendeeBadgeExport.as_view(),
        name='attendee_admin_export_badges'),
    re_path(r'^attendees/admin/export/food/$', FoodExport.as_view(),
       name='attendee_admin_export_food'),
    re_path(r'^attendees/admin/export/special_diets/'
        r'(?P<date>[0-9-]+)/(?P<meal>[a-z]+)/$', SpecialDietExport.as_view()),
    re_path(r'^attendees/admin/export/child_care/$', ChildCareExport.as_view(),
        name='attendee_admin_export_childcare'),
    re_path(r'^attendees/admin/export/bursaries/$', BursaryExport.as_view(),
       name='attendee_admin_export_bursaries'),
    re_path(r'^talks/admin/export/$', TalksExport.as_view(),
        name='talks_admin_export'),
    re_path(r'^talks/admin/export/speakers-registration/$',
        SpeakersRegistrationExport.as_view(),
        name='talks_speakers_registration'),
    re_path(r'^attendees/admin/export/fingerprints/$', FingerprintExport.as_view(),
        name='attendee_admin_export_fingerprints'),
    re_path(r'^attendees/admin/export/invoices/$', InvoiceExport.as_view(),
        name='attendee_admin_export_invoices'),
    re_path(r'^attendees/admin/export/security-info/$', SecurityInfoExport.as_view(),
        name='attendee_admin_export_security_info'),
    re_path(r'^attendees/admin/export/visas/$', VisaExport.as_view(),
        name='attendee_admin_export_visas'),
]

exports = [
    {
        'url': u.name,
        'name': u.callback.view_class.name,
        'permission_required': u.callback.view_class.permission_required,
    }
    for u in urlpatterns if u.name
]



from django.core.management.base import BaseCommand
from django.conf import settings

from register.dates import nights, meals
from register.models import AccommNight, Meal


class Command(BaseCommand):
    help = 'Create Meal and Night objects in the DB'

    def handle(self, *args, **options):
        for night in nights(orga=True):
            AccommNight.objects.get_or_create(date=night)
        skipped_meals = set(
            (meal, date)
            for (meal, date) in getattr(settings, 'DEBCONF_SKIPPED_MEALS', ())
        )
        for meal, date in meals(orga=True):
            if (meal, date) not in skipped_meals:
                Meal.objects.get_or_create(meal=meal, date=date)



# Generated by Django 4.2.14 on 2024-07-25 02:16

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("register", "0016_attendee_affiliation"),
    ]

    operations = [
        migrations.CreateModel(
            name="TravelInsurance",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("surname", models.CharField(max_length=100)),
                ("given_names", models.CharField(max_length=100)),
                ("country", models.CharField(max_length=2)),
                ("passport_no", models.CharField(max_length=100)),
                ("gender", models.CharField(max_length=20)),
                ("date_of_birth", models.DateField()),
                (
                    "attendee",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="travel_insurance",
                        to="register.attendee",
                    ),
                ),
            ],
        ),
    ]



from collections import OrderedDict, defaultdict
import datetime
from itertools import repeat
import logging

from django.conf import settings
from django.contrib.auth import get_user_model
from django.contrib.auth.mixins import PermissionRequiredMixin
from django.db import transaction
from django.urls import reverse
from django.shortcuts import redirect
from django.views.generic.base import TemplateView, View
from django.views.generic.detail import SingleObjectMixin
from django.views.generic.edit import FormView

from invoices.prices import invoice_user, meal_prices
from invoices.models import Invoice
from register.dates import (
    ARRIVE_ON_OR_AFTER, LEAVE_ON_OR_BEFORE, T_SHIRT_SWAP_ON_OR_AFTER,
    parse_date, meals as dc_meals)
from register.models import Accomm, Attendee, Food, Meal

from front_desk.forms import (
    CashInvoicePaymentForm, CheckInForm, CheckOutForm, FoodForm,
    RegisterOnSiteForm, SearchForm, ShoesForm, TShirtForm)
from front_desk.models import CheckIn

from exports.urls import exports

log = logging.getLogger(__name__)
shirt_sizes = dict(settings.DEBCONF_T_SHIRT_SIZES)


class CheckInPermissionMixin(PermissionRequiredMixin):
    permission_required = 'front_desk.change_checkin'


class CashBoxPermissionMixin(PermissionRequiredMixin):
    permission_required = 'invoices.change_invoice'


class Dashboard(PermissionRequiredMixin, TemplateView):
    template_name = 'front_desk/dashboard.html'
    permission_required = 'register.view_attendee'

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        form = SearchForm(data=self.request.GET)

        q = None
        if form.is_valid():
            q = form.cleaned_data['q']

        results = self.search_attendees(q)
        # Strip duplicates while maintaining order
        results = list(OrderedDict(zip(results, repeat(None))))

        context.update({
            'view': self,
            'form': form,
            'results': results[:20],
            'num_results': len(results),
            'exports': [
                export for export in exports
                if self.request.user.has_perm(export["permission_required"])
            ],
            "DEBCONF_REVIEW_FREE_ATTENDEES": settings.DEBCONF_REVIEW_FREE_ATTENDEES,
        })
        return context

    def search_attendees(self, query):
        if not query:
            return

        yield from Attendee.objects.filter(user__username=query)

        if '@' in query:
            yield from Attendee.objects.filter(user__email=query)

        if ' ' in query:
            first, last = query.split(' ', 1)
            yield from Attendee.objects.filter(
                user__first_name__iexact=first,
                user__last_name__iexact=last,
            )
        yield from Attendee.objects.filter(user__username__icontains=query)
        yield from Attendee.objects.filter(user__email__icontains=query)
        yield from Attendee.objects.filter(user__first_name__icontains=query)
        yield from Attendee.objects.filter(user__last_name__icontains=query)


class CheckInView(CheckInPermissionMixin, SingleObjectMixin, FormView):
    template_name = 'front_desk/check_in.html'
    form_class = CheckInForm

    model = Attendee
    slug_field = 'user__username'
    slug_url_kwarg = 'username'

    def get(self, request, *args, **kwargs):
        self.object = self.get_object()
        return super().get(request, *args, **kwargs)

    @transaction.atomic
    def post(self, request, *args, **kwargs):
        self.object = self.get_object()
        return super().post(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        attendee = self.object
        initial_tshirt = {
            'size': attendee.t_shirt_size,
        }
        initial_shoes = {
            'size': attendee.shoe_size,
        }

        t_shirt = 'No T-Shirt'
        if attendee.t_shirt_size:
            t_shirt = shirt_sizes[attendee.t_shirt_size]

        try:
            accomm = attendee.accomm
        except Accomm.DoesNotExist:
            accomm = None

        dates = sorted(set(date for meal, date in dc_meals(orga=True)))
        meal_labels = ['Breakfast', 'Lunch', 'Dinner']

        try:
            food = attendee.food
        except Food.DoesNotExist:
            meals = None
        else:
            meals = []
            for meal, meal_label in Meal.MEALS.items():
                cur_meal = []
                meal_dates = [entry.date
                              for entry in food.meals.filter(meal=meal)]
                for d in dates:
                    cur_meal.append((d, d in meal_dates))
                meals.append((meal_label, cur_meal))

        today = datetime.date.today()
        context.update({
            'accomm': accomm,
            'dates': dates,
            'DEBCONF_BILLING_CURRENCY': settings.DEBCONF_BILLING_CURRENCY,
            'DEBCONF_LOCAL_CURRENCY': settings.DEBCONF_LOCAL_CURRENCY,
            'DEBCONF_SHOE_SIZES': settings.DEBCONF_SHOE_SIZES,
            'invoices': Invoice.objects.filter(recipient=attendee.user),
            'meal_labels': meal_labels,
            'meals': meals,
            't_shirt': t_shirt,
            'shoes': attendee.shoe_size,
            't_shirt_form': TShirtForm(
                initial=initial_tshirt,
                username=attendee.user.username),
            'shoes_form': ShoesForm(
                initial=initial_shoes,
                username=attendee.user.username),
            't_shirt_swap_available': T_SHIRT_SWAP_ON_OR_AFTER <= today,
            'T_SHIRT_SWAP_ON_OR_AFTER': T_SHIRT_SWAP_ON_OR_AFTER,
        })
        return context

    def get_initial(self):
        try:
            check_in = self.object.check_in
        except CheckIn.DoesNotExist:
            return {}
        return {
            't_shirt': check_in.t_shirt,
            'shoes': check_in.shoes,
            'nametag': check_in.nametag,
            'notes': check_in.notes,
            'room_key': check_in.room_key,
            'meal_vouchers': check_in.meal_vouchers,
            'bedding': check_in.bedding,
            'transit_card': check_in.transit_card,
            'swag': check_in.swag,
        }

    def form_valid(self, form):
        attendee = self.object

        check_in, created = CheckIn.objects.update_or_create(
            attendee=attendee, defaults=form.cleaned_data)
        attendee.check_in = check_in

        log.info('Checked-in %s. Room Key: %s',
                 attendee.user.username, check_in.room_key)

        return super().form_valid(form)

    def get_success_url(self):
        return reverse('front_desk')


class CheckOutView(CheckInPermissionMixin, SingleObjectMixin, FormView):
    template_name = 'front_desk/check_out.html'
    form_class = CheckOutForm

    model = Attendee
    slug_field = 'user__username'
    slug_url_kwarg = 'username'

    def get(self, request, *args, **kwargs):
        self.object = self.get_object()
        return super().get(request, *args, **kwargs)

    @transaction.atomic
    def post(self, request, *args, **kwargs):
        self.object = self.get_object()
        return super().post(request, *args, **kwargs)

    def get_initial(self):
        check_in = self.object.check_in
        return {
            'returned_key': check_in.returned_key,
            'returned_card': check_in.returned_card,
            'returned_bedding': check_in.returned_bedding,
            'notes': check_in.notes,
        }

    def form_valid(self, form):
        attendee = self.object

        data = form.cleaned_data
        data['checked_out'] = True

        check_in, created = CheckIn.objects.update_or_create(
            attendee=attendee, defaults=data)
        attendee.check_in = check_in

        log.info('Checked-Out %s. Returned Key: %s, Card: %s',
                 attendee.user.username, check_in.returned_key,
                 check_in.returned_card)

        return super().form_valid(form)

    def get_success_url(self):
        return reverse('front_desk')


class ChangeFoodView(CashBoxPermissionMixin, SingleObjectMixin, FormView):
    form_class = FoodForm
    template_name = 'front_desk/change_food.html'

    model = Attendee
    slug_field = 'user__username'
    slug_url_kwarg = 'username'

    def get(self, request, *args, **kwargs):
        self.object = self.get_object()
        self.username = self.kwargs['username']
        return super().get(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['prices'] = meal_prices()
        return context

    @transaction.atomic
    def post(self, request, *args, **kwargs):
        self.object = self.get_object()
        self.username = self.kwargs['username']
        return super().post(request, *args, **kwargs)

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        kwargs['instance'] = self.object
        try:
            food = self.object.food
        except Food.DoesNotExist:
            pass
        else:
            kwargs['initial'] = {
                'meals': [meal.form_name for meal in food.meals.all()],
                'conference_dinner': any(meal for meal in food.meals.all()
                                         if meal.conference_dinner)
            }
        return kwargs

    def get_success_url(self):
        return reverse('front_desk.check_in',
                       kwargs={'username': self.username})

    def form_valid(self, form):
        data = form.cleaned_data
        user = self.object.user
        Invoice.objects.filter(
            recipient=user, status='new').update(status='canceled')
        if data['meals']:
            food, created = Food.objects.get_or_create(attendee=self.object)
            stored_meals = set(food.meals.all())
            requested_meals = set()
            for meal in data['meals']:
                meal, date = meal.split('_')
                date = parse_date(date)
                requested_meals.add(Meal.objects.get(meal=meal, date=date))

            added_meals = requested_meals - stored_meals
            removed_meals = stored_meals - requested_meals
            food.meals.remove(*removed_meals)
            food.meals.add(*added_meals)
            log.info('Modified meals for %s: Adding %s. Removing %s',
                     self.username,
                     ', '.join(sorted(str(meal) for meal in added_meals)),
                     ', '.join(sorted(str(meal) for meal in removed_meals)))
            invoice_user(user, save=True)
        else:
            Food.objects.filter(attendee=self.object).delete()
            log.info('Cancelled all meals for %s', self.username)


        return super().form_valid(form)


class ChangeShirtView(CheckInPermissionMixin, FormView):
    form_class = TShirtForm

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        self.username = self.kwargs['username']
        kwargs['username'] = self.username
        return kwargs

    def get_success_url(self):
        return reverse('front_desk.check_in',
                       kwargs={'username': self.username})

    def update_size(self, attendee, size):
        log.info('T-Shirt swap for %s: %s -> %s',
                 self.username, attendee.t_shirt_size, size)

        attendee.t_shirt_size = size
        attendee.save()

    @transaction.atomic
    def form_valid(self, form):
        attendee = Attendee.objects.get(user__username=self.username)
        size = form.cleaned_data['size']
        self.update_size(attendee, size)
        return super().form_valid(form)


class ChangeShoesView(ChangeShirtView):
    form_class = ShoesForm

    def update_size(self, attendee, size):
        log.info('Shoe swap for %s: %s -> %s',
                 self.username, attendee.shoe_size, size)

        attendee.shoe_size = size
        attendee.save()


class RegisterOnSite(CheckInPermissionMixin, FormView):
    form_class = RegisterOnSiteForm
    template_name = 'front_desk/register.html'

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context.update({
            'ARRIVE_ON_OR_AFTER': ARRIVE_ON_OR_AFTER,
            'LEAVE_ON_OR_BEFORE': LEAVE_ON_OR_BEFORE,
        })
        return context

    @transaction.atomic
    def form_valid(self, form):
        username = form.cleaned_data['username']
        self.username = username
        email = form.cleaned_data['email']

        names = form.cleaned_data['name'].split(None, 1)
        if len(names) > 1:
            first_name, last_name = names
        else:
            first_name, last_name = names[0], ''

        user = get_user_model().objects.create_user(
            username=username, email=email, first_name=first_name,
            last_name=last_name)
        Attendee.objects.create(
            user=user, nametag_3=username, arrival=datetime.datetime.now(),
            departure=form.cleaned_data['departure'], announce_me=False,
            register_announce=False, register_discuss=False,
            final_dates=True, reconfirm=True)

        log.info('Registered on-site: %s', username)

        return super().form_valid(form)

    def get_success_url(self):
        return reverse('front_desk.check_in',
                       kwargs={'username': self.username})


class CashInvoicePayment(CashBoxPermissionMixin, SingleObjectMixin, FormView):
    form_class = CashInvoicePaymentForm
    template_name = 'front_desk/cash_invoice_payment.html'

    model = Invoice
    slug_field = 'reference_number'
    slug_url_kwarg = 'ref'

    def get(self, request, *args, **kwargs):
        self.object = self.get_object()
        return super().get(request, *args, **kwargs)

    @transaction.atomic
    def post(self, request, *args, **kwargs):
        self.object = self.get_object()
        return super().post(request, *args, **kwargs)

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        kwargs['instance'] = self.object
        return kwargs

    def form_valid(self, form):
        invoice = self.object
        invoice.transaction_id = 'Cash: {}'.format(
            form.cleaned_data['receipt_number'])
        invoice.status = 'paid'
        invoice.save()

        log.info('Recieved cash-payment for %s: %s',
                 invoice.recipient.username, invoice.transaction_id)

        return super().form_valid(form)

    def get_success_url(self):
        return reverse('front_desk.check_in',
                       kwargs={'username': self.object.recipient.username})


class RoomsView(CheckInPermissionMixin, TemplateView):
    template_name = 'front_desk/rooms.html'

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        rooms = defaultdict(list)
        for accomm in Accomm.objects.select_related(
                'attendee', 'attendee__user',
                'attendee__user__userprofile',
                'attendee__check_in').all():
            room = accomm.room
            if not room:
                room = None
            rooms[room].append(accomm.attendee)

        unassigned = rooms.pop(None, [])

        if rooms:
            num_beds = max(len(occupants) for room, occupants in rooms.items())
        else:
            num_beds = 0
        bed_names = ['Bed {}'.format(i+1) for i in range(num_beds)]

        for room, occupants in rooms.items():
            occupants += [None] * (num_beds - len(occupants))

        context.update({
            'rooms': sorted(rooms.items()),
            'num_beds': num_beds,
            'bed_names': bed_names,
            'unassigned': unassigned,
        })
        return context


class IssueInvoiceView(CheckInPermissionMixin, SingleObjectMixin, View):
    model = Attendee
    slug_field = 'user__username'
    slug_url_kwarg = 'username'

    @transaction.atomic
    def post(self, request, *args, **kwargs):
        user = self.get_object().user
        invoice_user(user, save=True)
        return redirect('front_desk.check_in', user.username)
